'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');

var MODIFIER;

(function (MODIFIER) {
  MODIFIER[MODIFIER["ADD"] = 0] = "ADD";
  MODIFIER[MODIFIER["REMOVE"] = 1] = "REMOVE";
  MODIFIER[MODIFIER["MERGE"] = 2] = "MERGE";
})(MODIFIER || (MODIFIER = {}));

function key(key) {
  if (!key || key.length <= 1) {
    return [key, MODIFIER.MERGE];
  }

  if (key[0] === '+' || key[0] === '-' || key[0] === '*' || key[0] === '\\') {
    return [key.substring(1), key[0] === '+' ? MODIFIER.ADD : key[0] === '-' ? MODIFIER.REMOVE : MODIFIER.MERGE];
  }

  return [key, MODIFIER.MERGE];
}

const isArray = o => Array.isArray(o);
const isObject = o => typeof o === 'object' && o !== null && !isArray(o);

function merge_value_to_array(array, _value) {
  if (typeof _value === 'string') {
    const [val, mod] = key(_value);

    if (mod === MODIFIER.REMOVE) {
      const i = array.indexOf(val);
      array.splice(i, 1);
    } else {
      array.push(val);
    }
  } else {
    array.push(_value);
  }
}

function merge_array(array, _value) {
  const val = isArray(_value) ? _value : [_value];

  for (const v of val) {
    merge_value_to_array(array, v);
  }
}

function merge_value_to_object(object, _key, _value) {
  const [k, mod] = key(_key);

  if (mod === MODIFIER.REMOVE) {
    object[k] = undefined;
  } else if (mod === MODIFIER.ADD || object[k] == null) {
    object[k] = _value;
  } else {
    if (isArray(object[k])) {
      merge_array(object[k], _value);
    } else if (isObject(object[k])) {
      merge_object(object[k], _value);
    } else {
      object[k] = _value;
    }
  }
}

function merge_object(object, _value) {
  if (!isObject(_value)) return;

  for (const k in _value) {
    if (Object.prototype.hasOwnProperty.call(_value, k)) {
      merge_value_to_object(object, k, _value[k]);
    }
  }
}

function merge(json, value) {
  if (!isObject(json)) return;
  merge_object(json, value);
}

function evaluate(options, original) {
  if (!isObject(options) || !isObject(original)) return;

  for (const k in options) {
    const [_k] = key(k);

    if (Object.prototype.hasOwnProperty.call(options, k) && Object.prototype.hasOwnProperty.call(original, _k)) {
      if (options[k] === '^') {
        options[k] = original[_k];
      } else {
        evaluate(options[k], original[_k]);
      }
    }
  }
}

async function manifest(manifest, {
  options,
  cwd
}) {
  const original = require(path.join(cwd, 'package.json'));

  evaluate(options, original);
  merge(manifest, options);
  return manifest;
}

exports.manifest = manifest;
//# sourceMappingURL=index.js.map
